package org.eco.mubisoft.good_and_cheap.application.security;

import lombok.RequiredArgsConstructor;
import org.eco.mubisoft.good_and_cheap.user.control.filter.UserAuthenticationFilter;
import org.eco.mubisoft.good_and_cheap.user.control.filter.UserAuthorizationFilter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

/**
 * <p><b>Custom WebSecurityConfigurerAdapter</b></p>
 * <p>Manage the security of the application, applying different ACLs to modify
 * the security of the application.</p>
 */
@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    /**
     *
     */
    private final UserDetailsService userDetailsService;
    /**
     *
     */
    private final BCryptPasswordEncoder passwordEncoder;

    /**
     *
     * @param auth
     * @throws Exception
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(userDetailsService).passwordEncoder(passwordEncoder);
    }

    /**
     *
     * @param http
     * @throws Exception
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        UserAuthenticationFilter authenticationFilter = new UserAuthenticationFilter(authenticationManagerBean());
        authenticationFilter.setFilterProcessesUrl("/login");

        // Disable the default
        http.csrf().disable();
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);

        // ACL list
        http.authorizeRequests()
                .antMatchers("/user/delete/**", "/user/view/**")
                .hasAuthority("ROLE_ADMIN");
        http.authorizeRequests()
                .antMatchers("/recipe/create/**")
                .hasAnyAuthority("ROLE_USER", "ROLE_ADMIN");
        http.authorizeRequests()
                        .antMatchers("/product/create/**")
                        .hasAnyAuthority("ROLE_VENDOR", "ROLE_ADMIN");

        http.authorizeRequests().anyRequest().permitAll();

        // Add filters
        http.addFilter(authenticationFilter);
        http.addFilterBefore(new UserAuthorizationFilter(), UsernamePasswordAuthenticationFilter.class);
    }

    /**
     *
     * @return
     * @throws Exception
     */
    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }
}
